/*
 * accel.c
 * Mostra les dades de l'accelerometre i les envia utilitzant el Datalogging.
 */

#include <pebble.h>

/* Identificador del log */
#define DATA_LOG_TAG_ACCEL 52

/* Creacio dels apuntadors de la finestra i el text que apareixen al display */
static Window *s_main_window;
static TextLayer *s_output_layer;

/* Variable utilitzada pel dataLogging */
static DataLoggingSessionRef s_log_ref;

/* Handler que gestiona les dades rebudes de l'accelerometre. */
static void data_handler(AccelData *data, uint32_t num_samples) {

  /* Buffer static */
  static char s_buffer[128];

  /* Creacio de l'string que mostrarem per pantalla */
  snprintf(s_buffer, sizeof(s_buffer),
    "X: %d\nY: %d\nZ: %d\n",
    data[0].x, data[0].y, data[0].z
  );

  /* Mostrem les dades */
  text_layer_set_text(s_output_layer, s_buffer);

  /* Enviament de les dades a l'smartphone fent servir datalogging */
  DataLoggingResult result = data_logging_log(s_log_ref, data, 1);
  if (result != DATA_LOGGING_SUCCESS) {
    APP_LOG(APP_LOG_LEVEL_ERROR, "Error datalogging: %d", data[0].x);
  }
}

/* Carrega la pantalla. */
static void main_window_load(Window *window) {

  Layer *window_layer = window_get_root_layer(window);
  GRect window_bounds = layer_get_bounds(window_layer);

  /* Creacio i configuracio de les caracteristiques del TextLayer que es mostra a la pantalla */
  s_output_layer = text_layer_create(GRect(35, 20, window_bounds.size.w - 10, window_bounds.size.h));
  text_layer_set_font(s_output_layer, fonts_get_system_font(FONT_KEY_GOTHIC_28));
  text_layer_set_overflow_mode(s_output_layer, GTextOverflowModeWordWrap);
  layer_add_child(window_layer, text_layer_get_layer(s_output_layer));
}

/* Destrueix el TextLayer de la pantalla */
static void main_window_unload(Window *window) {
  text_layer_destroy(s_output_layer);
}

/* Inicialitza l'aplicacio i tots els seus serveis.*/
static void init() {

  /* Crea la finestra principal del programa */
  s_main_window = window_create();

  window_set_window_handlers(s_main_window, (WindowHandlers) {
    .load = main_window_load,
    .unload = main_window_unload
  });
  window_stack_push(s_main_window, true);

  /* Creacio del dataLogging */
  s_log_ref = data_logging_create(DATA_LOG_TAG_ACCEL, DATA_LOGGING_BYTE_ARRAY, sizeof(AccelData), true);

  /* Subscripcio al servei de l'accelerometre */
  int num_samples = 1;
  accel_data_service_subscribe(num_samples, data_handler);

  /* Triem la frequencia de mostreig */
  accel_service_set_sampling_rate(ACCEL_SAMPLING_50HZ);

  text_layer_set_text(s_output_layer, "Carregant...");
}

/* Finalitza l'aplicació i els serveis actius. */
static void deinit() {

  /* Destrueix la finestra principal */
  window_destroy(s_main_window);

  /* Elimina la subscripcio al servei de l'accelerometre */
  accel_data_service_unsubscribe();

  /* Finalitza el proces del dataLogging */
  data_logging_finish(s_log_ref);
}

/* Funcio principal. Primer inicialitza el programa i tots els serveis,
 * despres es crida al loop que mante el programa en funcionament
 * i finalment es finalitza el programa i tots els serveis en marxa.
 */
int main(void) {

  init();
  app_event_loop();
  deinit();
}
