package dmc.provapebble;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.TimeZone;

public class AccelData {
    private static final String TAG = AccelData.class.getSimpleName();

    final private int x;
    final private int y;
    final private int z;

    private long timestamp = 0;
    final private boolean didVibrate;

    // Donem format a les dades rebudes de l'accelerometre
    public AccelData(byte[] data) {
        x = (data[0] & 0xff) | (data[1] << 8);
        y = (data[2] & 0xff) | (data[3] << 8);
        z = (data[4] & 0xff) | (data[5] << 8);
        didVibrate = data[6] != 0;

        for (int i = 0; i < 8; i++) {
            timestamp |= ((long)(data[i+7] & 0xff)) << (i * 8);
        }
    }

    public static List<AccelData> fromDataArray(byte[] data) {
        List<AccelData> accels = new ArrayList<AccelData>();
        for (int i = 0; i < data.length; i += 15) {
            accels.add(new AccelData(Arrays.copyOfRange(data, i, i + 15)));
        }
        return accels;
    }

    // Retorna el valor de l'acceleracio en X.
    public int getXaxis(){
        return x;
    }

    // Retorna el valor de l'acceleracio en Y.
    public int getYaxis(){
        return y;
    }

    // Retorna el valor de l'acceleracio en Z.
    public int getZaxis(){
        return z;
    }

    // Retorna el timestamp.
    public long getTimestamp() {
        return timestamp;
    }

    // Converteix el timestamp a la zona horaria adequada.
    public void applyTimezone(TimeZone tz) {
        timestamp -= tz.getOffset(timestamp);
    }
}
