package dmc.provapebble;

import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.AsyncTask;
import android.support.v7.app.ActionBarActivity;
import android.os.Bundle;
import android.util.Log;
import android.content.Context;
import android.widget.TextView;
import android.widget.Toast;

import com.getpebble.android.kit.PebbleKit;
import com.getpebble.android.kit.PebbleKit.PebbleDataLogReceiver;

import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONObject;

import java.util.TimeZone;
import java.util.UUID;

public class MainActivity extends ActionBarActivity {

    // Es declara la UUID de l'aplicacio del rellotge per assegurar que les dades es transmeten
    private static final UUID WATCHAPP_UUID = UUID.fromString("906ff33f-1054-461f-9286-477ab44311db");

    // Es defineix el log que es fa servir
    private static final int DATA_LOG_TAG_ACCEL = 52;

    // Es defineix la zona horaria per tal de poder convertir correctament el timestamp rebut
    private static TimeZone tz = TimeZone.getTimeZone("Europe/Madrid");

    // Es crea el dataLogging
    private PebbleDataLogReceiver dataloggingReceiver;

    private StringBuilder resultBuilder = new StringBuilder();

    // Inicialitza l'Activity, crea la vista i estableix el text que apareix per pantalla.
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        TextView textView = (TextView) findViewById(R.id.text_estat);

        if (isConnected()){
            textView.setText("En funcionament");
        }
        else{
            textView.setText("No hi ha connexió a internet!");
        }
    }

    // Guarda l'estat de l'Activity abans de ser destruida.
    @Override
    protected void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        final TextView textView = (TextView)findViewById(R.id.text_estat);
        String stringBuilder = (String) textView.getText();
        outState.putString("resultBuilder", stringBuilder);
    }

    // Recupera l'estat previament guardat un cop es reinicia l'Activity.
    @Override
    protected void onRestoreInstanceState(Bundle savedInstanceState) {
        super.onRestoreInstanceState(savedInstanceState);
        final TextView textView = (TextView)findViewById(R.id.text_estat);
        String stringBuilder = savedInstanceState.getString("resultBuilder");
        textView.setText(stringBuilder);
    }

    // Es reben les dades de l'accelerometre, s'utilitza la classe AccelData
    // per separar cadascun dels elements de la mostra i es guarden.
    // Tambe inicialitza la classe HttpAsyncTask en finalitzar la sessio.
    @Override
    protected void onResume() {
        super.onResume();

        PebbleDataLogReceiver dataloggingReceiver = new PebbleDataLogReceiver(WATCHAPP_UUID) {

            @Override
            public void receiveData(Context context, UUID logUuid, Long timestamp, Long tag, byte [] data) {
                if(tag.intValue() == DATA_LOG_TAG_ACCEL) {
                    AccelData accels = new AccelData(data);
                    accels.applyTimezone(tz);

                    String string = accels.getTimestamp() + "," + accels.getXaxis() + "," + accels.getYaxis() + "," + accels.getZaxis() + "\n";
                    delete(resultBuilder,string);
                    resultBuilder.append(string);
                }
            }

            @Override
            public void onFinishSession(Context context, UUID logUuid, Long timestamp, Long tag) {
                super.onFinishSession(context, logUuid, timestamp, tag);
                if (tag.intValue() == DATA_LOG_TAG_ACCEL) {
                    TextView textView = (TextView) findViewById(R.id.text_estat);
                    textView.setText("Aplicació Pebble tancada!");
                    new HttpAsyncTask().execute("http://damirpi.ddns.net:8000");
                }
            }
        };

        PebbleKit.registerDataLogReceiver(this, dataloggingReceiver);
    }

    // Mantenir l'estat de l'activitat quan aquesta s'executa en segon pla.
    @Override
    protected void onPause() {
        super.onPause();

        if(dataloggingReceiver != null) {
            unregisterReceiver(dataloggingReceiver);
        }
    }

    // Esborra la cadena de caracters on estan representades les dades de l'accelerometre
    public void delete(StringBuilder sb, String s) {
        int start = sb.indexOf(s);
        if(start < 0)
            return;

        sb.delete(start, start + s.length());
    }

    // Obre una comunicacio HTTP, crea l'objecte que s'enviara en format JSON i l'envia
    // mitjançant el metode POST.
    public static String POST(String url, StringBuilder resultBuilder){
        int inputStream ;
        String result = "";
        try{
            HttpClient httpClient = new DefaultHttpClient();
            HttpPost httpPost = new HttpPost(url);

            String json="";
            JSONObject jsonObject = new JSONObject();
            jsonObject.accumulate("data", resultBuilder.toString());
            json = jsonObject.toString();

            StringEntity se = new StringEntity(json);
            httpPost.setEntity(se);
            httpPost.setHeader("Accept", "application/json");
            httpPost.setHeader("Content-type", "application/json");
            HttpResponse httpResponse = httpClient.execute(httpPost);
            inputStream = httpResponse.getStatusLine().getStatusCode();

            if (inputStream == 200)
                result = "Dades enviades!";
            else
                result = "ERROR en la transmissió de les dades!";
        } catch (Exception e){
            Log.d("Error", e.getLocalizedMessage());
        }

        return result;
    }

    // Comprova si hi ha connexio a la xarxa.
    public boolean isConnected(){
        ConnectivityManager connectivityManager = (ConnectivityManager) getSystemService(ActionBarActivity.CONNECTIVITY_SERVICE);
        NetworkInfo networkInfo = connectivityManager.getActiveNetworkInfo();
        if (networkInfo != null && networkInfo.isConnected())
            return true;
        else
            return false;
    }

    public class HttpAsyncTask extends AsyncTask<String, Void, String> {

        // Realitza la connexio HTTP.
        @Override
        protected String doInBackground(String... params) {
            return POST(params[0],resultBuilder);
        }

        // Indica si les dades han estat enviades correctament.
        @Override
        protected void onPostExecute(String s) {
            Toast.makeText(getApplicationContext(), s, Toast.LENGTH_LONG).show();
            resultBuilder.setLength(0);
            TextView textView = (TextView) findViewById(R.id.text_estat);
            textView.setText("En funcionament");
        }

    }
}
